@@ -3,14 +3,14 @@ package POE::Component::Server::Stomp;
 
 use POE::Session;
 use POE::Component::Server::TCP;
-use POE::Filter::Line;
+use POE::Filter::Stream;
 use IO::String;
 use Net::Stomp::Frame;
-
 use Carp qw(croak);
+use vars qw($VERSION);
 use strict;
 
-my $VERSION = '0.2.1';
+$VERSION = '0.2.2';
 
 sub new
 {
@@ -45,35 +45,8 @@ sub new
 	{
 		my ($kernel, $input) = @_[ KERNEL, ARG0 ];
 
-		my $io = IO::String->new( $input );
-
-		my $command;
-		my $headers;
-		my $body;
-
-		# read the command
-		$command = $io->getline;
-		chop $command;
-
-		# read headers
-		while (1)
-		{
-			my $line = $io->getline;
-			chop $line;
-			last if $line eq "";
-			my ( $key, $value ) = split /: ?/, $line, 2;
-			$headers->{$key} = $value;
-		}
-
-		# read the body (all the remaining data)
-		$io->read( $body, (length($input) - $io->tell()) );
-
 		# create the frame.
-		my $frame = Net::Stomp::Frame->new({
-			command => $command,
-			headers => $headers,
-			body    => $body
-		});
+		my $frame = _parse_frame( $input );
 
 		# Replace ARG0 with the parsed frame.
 		splice(@_, ARG0, 1, $frame);
@@ -84,7 +57,11 @@ sub new
 
 	# create the TCP server.
 	POE::Component::Server::TCP->new(
-		Port => 61613,
+		Alias    => $alias,
+		Address  => $address,
+		Hostname => $hname,
+		Port     => $port,
+		Domain   => $domain,
 
 		ClientInput        => $client_input,
 		ClientError        => $client_error,
@@ -97,6 +74,43 @@ sub new
 	return undef;
 }
 
+sub _parse_frame
+{
+	my ($input) = @_;
+
+	my $io = IO::String->new( $input );
+
+	my $command;
+	my $headers;
+	my $body;
+
+	# read the command
+	$command = $io->getline;
+	chop $command;
+
+	# read headers
+	while (1)
+	{
+		my $line = $io->getline;
+		chop $line;
+		last if $line eq "";
+		my ( $key, $value ) = split /: ?/, $line, 2;
+		$headers->{$key} = $value;
+	}
+
+	# read the body (all the remaining data)
+	$io->read( $body, (length($input) - $io->tell()) );
+
+	# create the frame.
+	my $frame = Net::Stomp::Frame->new({
+		command => $command,
+		headers => $headers,
+		body    => $body
+	});
+
+	return $frame;
+}
+
 1;
 
 __END__
